==> v0.5.2761.0.zip: terminal/src/cascadia/TerminalApp/AppKeyBindings.cpp <==
﻿// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.

#include "pch.h"
#include "AppKeyBindings.h"
#include "KeyChordSerialization.h"

#include "AppKeyBindings.g.cpp"

using namespace winrt::Microsoft::Terminal;
using namespace winrt::TerminalApp;

namespace winrt::TerminalApp::implementation
{
    void AppKeyBindings::SetKeyBinding(const TerminalApp::ShortcutAction& action,
                                       const Settings::KeyChord& chord)
    {
        _keyShortcuts[chord] = action;
    }

    // Method Description:
    // - Remove the action that's bound to a particular KeyChord.
    // Arguments:
    // - chord: the keystroke to remove the action for.
    // Return Value:
    // - <none>
    void AppKeyBindings::ClearKeyBinding(const Settings::KeyChord& chord)
    {
        _keyShortcuts.erase(chord);
    }

    Microsoft::Terminal::Settings::KeyChord AppKeyBindings::GetKeyBinding(TerminalApp::ShortcutAction const& action)
    {
        for (auto& kv : _keyShortcuts)
        {
            if (kv.second == action)
            {
                return kv.first;
            }
        }
        return { nullptr };
    }

    bool AppKeyBindings::TryKeyChord(const Settings::KeyChord& kc)
    {
        const auto keyIter = _keyShortcuts.find(kc);
        if (keyIter != _keyShortcuts.end())
        {
            const auto action = keyIter->second;
            return _DoAction(action);
        }
        return false;
    }

    bool AppKeyBindings::_DoAction(ShortcutAction action)
    {
        switch (action)
        {
        case ShortcutAction::CopyText:
        {
            auto args = winrt::make_self<CopyTextArgs>();
            args->TrimWhitespace(true);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _CopyTextHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::CopyTextWithoutNewlines:
        {
            auto args = winrt::make_self<CopyTextArgs>();
            args->TrimWhitespace(false);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _CopyTextHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::PasteText:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _PasteTextHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::NewTab:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _NewTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::OpenNewTabDropdown:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _OpenNewTabDropdownHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::DuplicateTab:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _DuplicateTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::OpenSettings:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _OpenSettingsHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }

        case ShortcutAction::NewTabProfile0:
        {
            auto args = winrt::make_self<NewTabWithProfileArgs>();
            args->ProfileIndex(0);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _NewTabWithProfileHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::NewTabProfile1:
        {
            auto args = winrt::make_self<NewTabWithProfileArgs>();
            args->ProfileIndex(1);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _NewTabWithProfileHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::NewTabProfile2:
        {
            auto args = winrt::make_self<NewTabWithProfileArgs>();
            args->ProfileIndex(2);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _NewTabWithProfileHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::NewTabProfile3:
        {
            auto args = winrt::make_self<NewTabWithProfileArgs>();
            args->ProfileIndex(3);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _NewTabWithProfileHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::NewTabProfile4:
        {
            auto args = winrt::make_self<NewTabWithProfileArgs>();
            args->ProfileIndex(4);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _NewTabWithProfileHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::NewTabProfile5:
        {
            auto args = winrt::make_self<NewTabWithProfileArgs>();
            args->ProfileIndex(5);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _NewTabWithProfileHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::NewTabProfile6:
        {
            auto args = winrt::make_self<NewTabWithProfileArgs>();
            args->ProfileIndex(6);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _NewTabWithProfileHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::NewTabProfile7:
        {
            auto args = winrt::make_self<NewTabWithProfileArgs>();
            args->ProfileIndex(7);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _NewTabWithProfileHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::NewTabProfile8:
        {
            auto args = winrt::make_self<NewTabWithProfileArgs>();
            args->ProfileIndex(8);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _NewTabWithProfileHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }

        case ShortcutAction::NewWindow:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _NewWindowHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::CloseWindow:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _CloseWindowHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::CloseTab:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _CloseTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::ClosePane:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _ClosePaneHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }

        case ShortcutAction::ScrollUp:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _ScrollUpHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::ScrollDown:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _ScrollDownHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::ScrollUpPage:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _ScrollUpPageHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::ScrollDownPage:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _ScrollDownPageHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }

        case ShortcutAction::NextTab:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _NextTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::PrevTab:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _PrevTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }

        case ShortcutAction::SplitVertical:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _SplitVerticalHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::SplitHorizontal:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _SplitHorizontalHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }

        case ShortcutAction::SwitchToTab0:
        {
            auto args = winrt::make_self<SwitchToTabArgs>();
            args->TabIndex(0);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _SwitchToTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::SwitchToTab1:
        {
            auto args = winrt::make_self<SwitchToTabArgs>();
            args->TabIndex(1);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _SwitchToTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::SwitchToTab2:
        {
            auto args = winrt::make_self<SwitchToTabArgs>();
            args->TabIndex(2);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _SwitchToTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::SwitchToTab3:
        {
            auto args = winrt::make_self<SwitchToTabArgs>();
            args->TabIndex(3);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _SwitchToTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::SwitchToTab4:
        {
            auto args = winrt::make_self<SwitchToTabArgs>();
            args->TabIndex(4);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _SwitchToTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::SwitchToTab5:
        {
            auto args = winrt::make_self<SwitchToTabArgs>();
            args->TabIndex(5);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _SwitchToTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::SwitchToTab6:
        {
            auto args = winrt::make_self<SwitchToTabArgs>();
            args->TabIndex(6);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _SwitchToTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::SwitchToTab7:
        {
            auto args = winrt::make_self<SwitchToTabArgs>();
            args->TabIndex(7);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _SwitchToTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::SwitchToTab8:
        {
            auto args = winrt::make_self<SwitchToTabArgs>();
            args->TabIndex(8);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _SwitchToTabHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::ResizePaneLeft:
        {
            auto args = winrt::make_self<ResizePaneArgs>();
            args->Direction(Direction::Left);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _ResizePaneHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::ResizePaneRight:
        {
            auto args = winrt::make_self<ResizePaneArgs>();
            args->Direction(Direction::Right);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _ResizePaneHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::ResizePaneUp:
        {
            auto args = winrt::make_self<ResizePaneArgs>();
            args->Direction(Direction::Up);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _ResizePaneHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::ResizePaneDown:
        {
            auto args = winrt::make_self<ResizePaneArgs>();
            args->Direction(Direction::Down);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _ResizePaneHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::MoveFocusLeft:
        {
            auto args = winrt::make_self<MoveFocusArgs>();
            args->Direction(Direction::Left);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _MoveFocusHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::MoveFocusRight:
        {
            auto args = winrt::make_self<MoveFocusArgs>();
            args->Direction(Direction::Right);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _MoveFocusHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::MoveFocusUp:
        {
            auto args = winrt::make_self<MoveFocusArgs>();
            args->Direction(Direction::Up);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _MoveFocusHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::MoveFocusDown:
        {
            auto args = winrt::make_self<MoveFocusArgs>();
            args->Direction(Direction::Down);
            auto eventArgs = winrt::make_self<ActionEventArgs>(*args);
            _MoveFocusHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        default:
            return false;
        }
        return false;
    }

    // Method Description:
    // - Takes the KeyModifier flags from Terminal and maps them to the WinRT types which are used by XAML
    // Return Value:
    // - a Windows::System::VirtualKeyModifiers object with the flags of which modifiers used.
    Windows::System::VirtualKeyModifiers AppKeyBindings::ConvertVKModifiers(Settings::KeyModifiers modifiers)
    {
        Windows::System::VirtualKeyModifiers keyModifiers = Windows::System::VirtualKeyModifiers::None;

        if (WI_IsFlagSet(modifiers, Settings::KeyModifiers::Ctrl))
        {
            keyModifiers |= Windows::System::VirtualKeyModifiers::Control;
        }
        if (WI_IsFlagSet(modifiers, Settings::KeyModifiers::Shift))
        {
            keyModifiers |= Windows::System::VirtualKeyModifiers::Shift;
        }
        if (WI_IsFlagSet(modifiers, Settings::KeyModifiers::Alt))
        {
            // note: Menu is the Alt VK_MENU
            keyModifiers |= Windows::System::VirtualKeyModifiers::Menu;
        }

        return keyModifiers;
    }
}

==> v0.7.3382.0.zip: terminal/src/cascadia/TerminalApp/AppKeyBindings.cpp <==
﻿// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.

#include "pch.h"
#include "AppKeyBindings.h"
#include "KeyChordSerialization.h"

#include "AppKeyBindings.g.cpp"

using namespace winrt::Microsoft::Terminal;
using namespace winrt::TerminalApp;
using namespace winrt::Microsoft::Terminal::Settings;

namespace winrt::TerminalApp::implementation
{
    void AppKeyBindings::SetKeyBinding(const TerminalApp::ActionAndArgs& actionAndArgs,
                                       const Settings::KeyChord& chord)
    {
        _keyShortcuts[chord] = actionAndArgs;
    }

    // Method Description:
    // - Remove the action that's bound to a particular KeyChord.
    // Arguments:
    // - chord: the keystroke to remove the action for.
    // Return Value:
    // - <none>
    void AppKeyBindings::ClearKeyBinding(const Settings::KeyChord& chord)
    {
        _keyShortcuts.erase(chord);
    }

    KeyChord AppKeyBindings::GetKeyBindingForAction(TerminalApp::ShortcutAction const& action)
    {
        for (auto& kv : _keyShortcuts)
        {
            if (kv.second.Action() == action)
            {
                return kv.first;
            }
        }
        return { nullptr };
    }

    // Method Description:
    // - Lookup the keychord bound to a particular combination of ShortcutAction
    //   and IActionArgs. This enables searching no only for the binding of a
    //   particular ShortcutAction, but also a particular set of values for
    //   arguments to that action.
    // Arguments:
    // - actionAndArgs: The ActionAndArgs to lookup the keybinding for.
    // Return Value:
    // - The bound keychord, if this ActionAndArgs is bound to a key, otherwise nullptr.
    KeyChord AppKeyBindings::GetKeyBindingForActionWithArgs(TerminalApp::ActionAndArgs const& actionAndArgs)
    {
        for (auto& kv : _keyShortcuts)
        {
            if (kv.second.Action() == actionAndArgs.Action() &&
                kv.second.Args().Equals(actionAndArgs.Args()))
            {
                return kv.first;
            }
        }
        return { nullptr };
    }

    bool AppKeyBindings::TryKeyChord(const Settings::KeyChord& kc)
    {
        const auto keyIter = _keyShortcuts.find(kc);
        if (keyIter != _keyShortcuts.end())
        {
            const auto actionAndArgs = keyIter->second;
            return _DoAction(actionAndArgs);
        }
        return false;
    }

    bool AppKeyBindings::_DoAction(ActionAndArgs actionAndArgs)
    {
        const auto& action = actionAndArgs.Action();
        const auto& args = actionAndArgs.Args();
        auto eventArgs = args ? winrt::make_self<ActionEventArgs>(args) :
                                winrt::make_self<ActionEventArgs>();

        switch (action)
        {
        case ShortcutAction::CopyText:
        {
            _CopyTextHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::CopyTextWithoutNewlines:
        {
            _CopyTextHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::PasteText:
        {
            _PasteTextHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::OpenNewTabDropdown:
        {
            _OpenNewTabDropdownHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::DuplicateTab:
        {
            _DuplicateTabHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::OpenSettings:
        {
            _OpenSettingsHandlers(*this, *eventArgs);
            break;
        }

        case ShortcutAction::NewTab:
        case ShortcutAction::NewTabProfile0:
        case ShortcutAction::NewTabProfile1:
        case ShortcutAction::NewTabProfile2:
        case ShortcutAction::NewTabProfile3:
        case ShortcutAction::NewTabProfile4:
        case ShortcutAction::NewTabProfile5:
        case ShortcutAction::NewTabProfile6:
        case ShortcutAction::NewTabProfile7:
        case ShortcutAction::NewTabProfile8:
        {
            _NewTabHandlers(*this, *eventArgs);
            break;
        }

        case ShortcutAction::NewWindow:
        {
            _NewWindowHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::CloseWindow:
        {
            _CloseWindowHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::CloseTab:
        {
            _CloseTabHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::ClosePane:
        {
            _ClosePaneHandlers(*this, *eventArgs);
            break;
        }

        case ShortcutAction::ScrollUp:
        {
            _ScrollUpHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::ScrollDown:
        {
            _ScrollDownHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::ScrollUpPage:
        {
            _ScrollUpPageHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::ScrollDownPage:
        {
            _ScrollDownPageHandlers(*this, *eventArgs);
            break;
        }

        case ShortcutAction::NextTab:
        {
            _NextTabHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::PrevTab:
        {
            _PrevTabHandlers(*this, *eventArgs);
            break;
        }

        case ShortcutAction::SplitVertical:
        {
            _SplitVerticalHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::SplitHorizontal:
        {
            _SplitHorizontalHandlers(*this, *eventArgs);
            break;
        }

        case ShortcutAction::SwitchToTab:
        case ShortcutAction::SwitchToTab0:
        case ShortcutAction::SwitchToTab1:
        case ShortcutAction::SwitchToTab2:
        case ShortcutAction::SwitchToTab3:
        case ShortcutAction::SwitchToTab4:
        case ShortcutAction::SwitchToTab5:
        case ShortcutAction::SwitchToTab6:
        case ShortcutAction::SwitchToTab7:
        case ShortcutAction::SwitchToTab8:
        {
            _SwitchToTabHandlers(*this, *eventArgs);
            break;
        }

        case ShortcutAction::ResizePane:
        case ShortcutAction::ResizePaneLeft:
        case ShortcutAction::ResizePaneRight:
        case ShortcutAction::ResizePaneUp:
        case ShortcutAction::ResizePaneDown:
        {
            _ResizePaneHandlers(*this, *eventArgs);
            break;
        }

        case ShortcutAction::MoveFocus:
        case ShortcutAction::MoveFocusLeft:
        case ShortcutAction::MoveFocusRight:
        case ShortcutAction::MoveFocusUp:
        case ShortcutAction::MoveFocusDown:
        {
            _MoveFocusHandlers(*this, *eventArgs);
            break;
        }

        case ShortcutAction::IncreaseFontSize:
        {
            _AdjustFontSizeHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::DecreaseFontSize:
        {
            _AdjustFontSizeHandlers(*this, *eventArgs);
            break;
        }
        case ShortcutAction::ResetFontSize:
        {
            auto eventArgs = winrt::make_self<ActionEventArgs>();
            _ResetFontSizeHandlers(*this, *eventArgs);
            return eventArgs->Handled();
        }
        case ShortcutAction::ToggleFullscreen:
        {
            _ToggleFullscreenHandlers(*this, *eventArgs);
            break;
        }
        default:
            return false;
        }
        return eventArgs->Handled();
    }

    // Method Description:
    // - Takes the KeyModifier flags from Terminal and maps them to the WinRT types which are used by XAML
    // Return Value:
    // - a Windows::System::VirtualKeyModifiers object with the flags of which modifiers used.
    Windows::System::VirtualKeyModifiers AppKeyBindings::ConvertVKModifiers(Settings::KeyModifiers modifiers)
    {
        Windows::System::VirtualKeyModifiers keyModifiers = Windows::System::VirtualKeyModifiers::None;

        if (WI_IsFlagSet(modifiers, Settings::KeyModifiers::Ctrl))
        {
            keyModifiers |= Windows::System::VirtualKeyModifiers::Control;
        }
        if (WI_IsFlagSet(modifiers, Settings::KeyModifiers::Shift))
        {
            keyModifiers |= Windows::System::VirtualKeyModifiers::Shift;
        }
        if (WI_IsFlagSet(modifiers, Settings::KeyModifiers::Alt))
        {
            // note: Menu is the Alt VK_MENU
            keyModifiers |= Windows::System::VirtualKeyModifiers::Menu;
        }

        return keyModifiers;
    }
}
